@@ -22,11 +22,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xpad-settings.h"
 #include "fio.h"
 
-G_DEFINE_TYPE(XpadSettings, xpad_settings, G_TYPE_OBJECT)
-#define XPAD_SETTINGS_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), XPAD_TYPE_SETTINGS, XpadSettingsPrivate))
-
-#define DEFAULTS_FILENAME	"default-style"
-
 struct XpadSettingsPrivate 
 {
 	guint width;
@@ -35,6 +30,7 @@ struct XpadSettingsPrivate
 	gboolean confirm_destroy;
 	gboolean edit_lock;
 	gboolean sticky;
+	guint tray_click_configuration;
 	gboolean has_toolbar;
 	gboolean autohide_toolbar;
 	gboolean has_scrollbar;
@@ -44,6 +40,10 @@ struct XpadSettingsPrivate
 	GSList *toolbar_buttons;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE(XpadSettings, xpad_settings, G_TYPE_OBJECT)
+
+#define DEFAULTS_FILENAME	"default-style"
+
 enum
 {
 	CHANGE_BUTTONS,
@@ -59,6 +59,7 @@ enum
   PROP_CONFIRM_DESTROY,
   PROP_STICKY,
   PROP_EDIT_LOCK,
+  PROP_TRAY_CLICK_CONFIGURATION,
   PROP_HAS_TOOLBAR,
   PROP_AUTOHIDE_TOOLBAR,
   PROP_HAS_SCROLLBAR,
@@ -72,6 +73,7 @@ static void load_from_file (XpadSettings *settings, const gchar *filename);
 static void save_to_file (XpadSettings *settings, const gchar *filename);
 static void xpad_settings_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void xpad_settings_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void xpad_settings_dispose (GObject *object);
 static void xpad_settings_finalize (GObject *object);
 
 static XpadSettings *_xpad_settings = NULL;
@@ -91,7 +93,8 @@ static void
 xpad_settings_class_init (XpadSettingsClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-	
+
+	gobject_class->dispose = xpad_settings_dispose;
 	gobject_class->finalize = xpad_settings_finalize;
 	gobject_class->set_property = xpad_settings_set_property;
 	gobject_class->get_property = xpad_settings_get_property;
@@ -149,7 +152,15 @@ xpad_settings_class_init (XpadSettingsClass *klass)
 	                                                       "Whether edit lock mode is enabled",
 	                                                       FALSE,
 	                                                       G_PARAM_READWRITE));
-	
+	g_object_class_install_property (gobject_class,
+                                     PROP_TRAY_CLICK_CONFIGURATION,
+                                     g_param_spec_uint ("tray_click_configuration",
+                                                        "Tray Click Configuration",
+                                                        "What configuration is selected on tray click",
+                                                        0,
+                                                        G_MAXUINT,
+                                                        2,
+                                                        G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_class,
 	                                 PROP_HAS_TOOLBAR,
 	                                 g_param_spec_boolean ("has_toolbar",
@@ -207,8 +218,6 @@ xpad_settings_class_init (XpadSettingsClass *klass)
 		              G_STRUCT_OFFSET (XpadSettingsClass, change_buttons),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
-	g_type_class_add_private (gobject_class, sizeof (XpadSettingsPrivate));
 }
 
 static void
@@ -216,7 +225,7 @@ xpad_settings_init (XpadSettings *settings)
 {
 	GdkColor back, text;
 	
-	settings->priv = XPAD_SETTINGS_GET_PRIVATE (settings);
+	settings->priv = xpad_settings_get_instance_private(settings);
 	
 	/* A pleasant light yellow color, similar to 
 	   commercial sticky notes. */
@@ -239,6 +248,7 @@ xpad_settings_init (XpadSettings *settings)
 	settings->priv->confirm_destroy = TRUE;
 	settings->priv->sticky = FALSE;
 	settings->priv->edit_lock = FALSE;
+	settings->priv->tray_click_configuration = 0;
 	settings->priv->fontname = NULL;
 	settings->priv->has_toolbar = TRUE;
 	settings->priv->autohide_toolbar = TRUE;
@@ -252,14 +262,24 @@ xpad_settings_init (XpadSettings *settings)
 	load_from_file (settings, DEFAULTS_FILENAME);
 }
 
+static void
+xpad_settings_dispose (GObject *object)
+{
+	G_OBJECT_CLASS (xpad_settings_parent_class)->dispose (object);
+}
+
 static void
 xpad_settings_finalize (GObject *object)
 {
 	XpadSettings *settings = XPAD_SETTINGS (object);
 	
 	g_slist_free (settings->priv->toolbar_buttons);
-	gdk_color_free (settings->priv->text);
-	gdk_color_free (settings->priv->back);
+
+	if (settings->priv->text)
+		gdk_color_free (settings->priv->text);
+	if (settings->priv->back)
+		gdk_color_free (settings->priv->back);
+
 	g_free (settings->priv->fontname);
 	
 	G_OBJECT_CLASS (xpad_settings_parent_class)->finalize (object);
@@ -350,6 +370,21 @@ gboolean xpad_settings_get_edit_lock (XpadSettings *settings)
 	return settings->priv->edit_lock;
 }
 
+void xpad_settings_set_tray_click_handler (XpadSettings *settings, guint conf)
+{
+	if (settings->priv->tray_click_configuration == conf)
+		return;
+	
+	settings->priv->tray_click_configuration = conf;
+	save_to_file(settings, DEFAULTS_FILENAME);
+	g_object_notify (G_OBJECT (settings), "tray_click_configuration");
+}
+
+guint xpad_settings_get_tray_click_handler(XpadSettings *settings)
+{
+	return settings->priv->tray_click_configuration;
+}
+
 void xpad_settings_set_has_toolbar (XpadSettings *settings, gboolean toolbar)
 {
 	if (settings->priv->has_toolbar == toolbar)
@@ -427,20 +462,27 @@ void xpad_settings_add_toolbar_button (XpadSettings *settings, const gchar *butt
 	g_signal_emit (settings, signals[CHANGE_BUTTONS], 0);
 }
 
+/* Unused function of previous developer
 gboolean xpad_settings_move_toolbar_button (XpadSettings *settings, gint button, gint new)
 {
-	GSList *element;
+	GSList *element = NULL;
 	gpointer data;
 	
-	if (button == new)
+	if (button == new) {
+		g_slist_free(element);
 		return FALSE;
+	}
 	
-	if (new >= g_slist_length (settings->priv->toolbar_buttons) || 0 > new )
+	if (new >= (gint) g_slist_length (settings->priv->toolbar_buttons) || 0 > new ) {
+		g_slist_free(element);
 		return FALSE;
+	}
 	
 	element = g_slist_nth (settings->priv->toolbar_buttons, button);
-	if (!element)
+	if (!element) {
+		g_slist_free(element);
 		return FALSE;
+	}
 	
 	data = element->data;
 	settings->priv->toolbar_buttons = g_slist_delete_link (settings->priv->toolbar_buttons, element);
@@ -450,25 +492,81 @@ gboolean xpad_settings_move_toolbar_button (XpadSettings *settings, gint button,
 	
 	g_signal_emit (settings, signals[CHANGE_BUTTONS], 0);
 	
+	g_slist_free(element);
+
 	return TRUE;
 }
+*/
 
+static void xpad_settings_remove_toolbar_list_element (XpadSettings *settings, GSList *element)
+{
+	g_free (element->data);
+	settings->priv->toolbar_buttons = g_slist_delete_link (settings->priv->toolbar_buttons, element);
+}
+
+/* Unused function of previous developer
 gboolean xpad_settings_remove_toolbar_button (XpadSettings *settings, gint button)
 {
 	GSList *element;
 	
 	element = g_slist_nth (settings->priv->toolbar_buttons, button);
 	
-	if (!element)
+	if (!element) {
+		g_slist_free(element);
 		return FALSE;
+	}
 	
-	g_free (element->data);
-	settings->priv->toolbar_buttons = g_slist_delete_link (settings->priv->toolbar_buttons, element);
+	xpad_settings_remove_toolbar_list_element (settings, element);
 	
 	save_to_file (settings, DEFAULTS_FILENAME);
 	
 	g_signal_emit (settings, signals[CHANGE_BUTTONS], 0);
 	
+	g_slist_free(element);
+
+	return TRUE;
+}
+*/
+
+gboolean xpad_settings_remove_all_toolbar_buttons (XpadSettings *settings)
+{
+	if (settings->priv->toolbar_buttons == NULL)
+		return FALSE;
+
+	while (settings->priv->toolbar_buttons)
+	{
+		g_free (settings->priv->toolbar_buttons->data);
+		settings->priv->toolbar_buttons = 
+			g_slist_delete_link (settings->priv->toolbar_buttons,
+					settings->priv->toolbar_buttons);
+	}
+
+	settings->priv->toolbar_buttons = NULL;
+
+	g_signal_emit (settings, signals[CHANGE_BUTTONS], 0);
+
+	return TRUE;
+}
+
+gboolean xpad_settings_remove_last_toolbar_button (XpadSettings *settings)
+{
+	GSList *element;
+	
+	element = g_slist_last (settings->priv->toolbar_buttons);
+	
+	if (!element) {
+		g_slist_free(element);
+		return FALSE;
+	}
+	
+	xpad_settings_remove_toolbar_list_element (settings, element);
+	
+	save_to_file (settings, DEFAULTS_FILENAME);
+	
+	g_signal_emit (settings, signals[CHANGE_BUTTONS], 0);
+	
+	g_slist_free(element);
+
 	return TRUE;
 }
 
@@ -564,7 +662,11 @@ xpad_settings_set_property (GObject *object, guint prop_id, const GValue *value,
 	case PROP_EDIT_LOCK:
 		xpad_settings_set_edit_lock (settings, g_value_get_boolean (value));
 		break;
-	
+		
+	case PROP_TRAY_CLICK_CONFIGURATION:
+		xpad_settings_set_tray_click_handler(settings, g_value_get_uint(value));
+		break;
+		
 	case PROP_HAS_TOOLBAR:
 		xpad_settings_set_has_toolbar (settings, g_value_get_boolean (value));
 		break;
@@ -684,6 +786,7 @@ load_from_file (XpadSettings *settings, const gchar *filename)
 		"b|confirm_destroy", &settings->priv->confirm_destroy,
 		"b|edit_lock", &settings->priv->edit_lock,
 		"b|sticky_on_start", &settings->priv->sticky,
+		"u|tray_click_configuration", &settings->priv->tray_click_configuration,
 		"h|back_red", &back.red,
 		"h|back_green", &back.green,
 		"h|back_blue", &back.blue,
@@ -705,13 +808,13 @@ load_from_file (XpadSettings *settings, const gchar *filename)
 		gdk_color_free (settings->priv->text);
 		settings->priv->text = gdk_color_copy (&text);
 	}
-	
+
+	gdk_color_free (settings->priv->back);
 	if (use_back)
-	{
-		gdk_color_free (settings->priv->back);
 		settings->priv->back = gdk_color_copy (&back);
-	}
-	
+	else
+		settings->priv->back = NULL;
+
 	if (settings->priv->fontname &&
 	    strcmp (settings->priv->fontname, "NULL") == 0)
 	{
@@ -775,6 +878,7 @@ save_to_file (XpadSettings *settings, const gchar *filename)
 		"b|confirm_destroy", settings->priv->confirm_destroy,
 		"b|edit_lock", settings->priv->edit_lock,
 		"b|sticky_on_start", settings->priv->sticky,
+		"u|tray_click_configuration", settings->priv->tray_click_configuration,
 		"h|back_red", settings->priv->back ? settings->priv->back->red : 0,
 		"h|back_green", settings->priv->back ? settings->priv->back->green : 0,
 		"h|back_blue", settings->priv->back ? settings->priv->back->blue : 0,
